Pettanr::Application.routes.draw do

  devise_for :users
  devise_for :admins
  devise_for :demand_users
  
  Pettanr::Application.licenses.each do |name, klass|
    mount Module.const_get(klass)::Engine => "/#{name}"
  end
  
  resources :authors do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      get :count
      post :create
      get :list
    end
    member do
      put :update
      delete :destroy
      get :browse
    end
  end
  resources :artists do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      get :count
      post :create
      get :list
    end
    member do
      put :update
      delete :destroy
      get :browse
    end
  end
  resources :speech_balloon_templates do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
      delete :destroy
    end
  end
  resources :speeches do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
    end
  end
  resources :balloons do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
    end
  end
  resources :speech_balloons do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
    end
  end
  resources :panel_pictures do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
    end
  end
  resources :pictures do
    collection do
      get :show
      get :credit
      get :search
      get :list
    end
    member do
      get :credit
      get :browse
    end
  end
  resources :resource_pictures do
    collection do
      get :index
      get :show
      get :new
      post :create
      get :count
      get :list
    end
    member do
      delete :destroy
      get :credit
      get :browse
    end
  end
  resources :original_pictures do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      post :create
      get :list
    end
    member do
      get :edit
      put :update
      delete :destroy
      get :history
      get :browse
    end
  end
  resources :original_picture_license_groups do
    new do
      post :new
    end
    collection do
      post :create
    end
  end
  resources :colors do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
    end
  end
  resources :panel_colors do
    collection do
      get :index
      get :show
    end
    member do
      get :browse
    end
  end
  resources :ground_colors do
    collection do
      get :index
      get :show
    end
    member do
      get :browse
    end
  end
  resources :ground_pictures do
    collection do
      get :index
      get :show
    end
    member do
      get :browse
    end
  end
  resources :panels do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      get :count
      post :create
      get :list
    end
    member do
      put :update
      delete :destroy
      get :browse
    end
  end
  resources :stories do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      post :create
      get :list
    end
    member do
      get :comic
      put :update
      delete :destroy
      get :browse
    end
  end
  resources :comics do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      get :count
      post :create
      get :list
    end
    member do
      get :edit
      put :update
      delete :destroy
      get :browse
    end
  end
  resources :licenses do
    collection do
      get :index
      get :show
      get :search
      get :list
    end
    member do
      get :browse
    end
  end
  resources :license_groups do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
      delete :destroy
    end
  end
  resources :system_pictures do
    collection do
      get :index
      get :show
      get :list
    end
    member do
      get :browse
      delete :destroy
    end
  end
  resources :provider_sources do
    collection do
      get :index
      post :import
      get :list
      get :browse
    end
    member do
    end
  end
  resources :providers do
    collection do
      get :index
      get :show
      get :list
      get :browse
    end
    member do
      delete :destroy
    end
  end
  resources :provider_statuses do
    collection do
      get :index
      get :show
      get :list
      get :browse
    end
    member do
      get :edit
      put :update
      delete :destroy
      post :licenses_import
      post :artists_import
      post :original_pictures_import
    end
  end
  resources :demanders do
    new do
      get :new
    end
    collection do
      get :index
      get :show
      post :create
      get :list
      get :browse
      post :req
      get :licenses_export
      get :artists_export
      get :original_pictures_export
      get :pictures_export
    end
    member do
      get :edit
      put :update
      delete :destroy
    end
  end
  resources :demander_statuses do
    collection do
      get :index
      get :show
      get :list
      get :browse
    end
    member do
      get :edit
      put :update
      delete :destroy
    end
  end
  # The priority is based upon order of creation:
  # first created -> highest priority.

  # Sample of regular route:
  #   match 'products/:id' => 'catalog#view'
  # Keep in mind you can assign values other than :controller and :action
  match 'resource_pictures/:subdir/:id(.:format)/' => 'resource_pictures#show'
  match 'pictures/:subdir/:id(.:format)/' => 'pictures#show'
  match 'stories/comic/:id(.:format)' => 'stories#comic'
  match 'home/(:action)', :controller => 'home'
  match 'system/(:action)', :controller => 'system'
  match 'authors/(:action(/:id))', :controller => 'authors'
  
  # Sample of named route:
  #   match 'products/:id/purchase' => 'catalog#purchase', :as => :purchase
  # This route can be invoked with purchase_url(:id => product.id)

  # Sample resource route (maps HTTP verbs to controller actions automatically):
  #   resources :products

  # Sample resource route with options:
  #   resources :products do
  #     member do
  #       get 'short'
  #       post 'toggle'
  #     end
  #
  #     collection do
  #       get 'sold'
  #     end
  #   end

  # Sample resource route with sub-resources:
  #   resources :products do
  #     resources :comments, :sales
  #     resource :seller
  #   end

  # Sample resource route with more complex sub-resources
  #   resources :products do
  #     resources :comments
  #     resources :sales do
  #       get 'recent', :on => :collection
  #     end
  #   end

  # Sample resource route within a namespace:
  #   namespace :admin do
  #     # Directs /admin/products/* to Admin::ProductsController
  #     # (app/controllers/admin/products_controller.rb)
  #     resources :products
  #   end

  # You can have the root of your site routed with "root"
  # just remember to delete public/index.html.
  root :to => 'comics#top'

  # See how all your routes lay out with "rake routes"

  # This is a legacy wild controller route that's not recommended for RESTful applications.
  # Note: This route will make all actions in every controller accessible via GET requests.
  #match ':controller(/:action(/:id(.:format)))'
end
