# -*- encoding: utf-8 -*-
#画像処理
require 'pettan_imager'

describe PettanImager do
  before do
    @f = File.dirname(__FILE__) + '/test.png'
    @d = File.open(@f, 'rb').read
  end

  describe 'クラス変数に於いて' do
    context 'サムネイル幅のとき' do
      it 'デフォルトは50' do
        r = PettanImager.tmb_w
        r.should eq 50
      end
      it 'セットできる' do
        PettanImager.tmb_w = 64
        r = PettanImager.tmb_w
        r.should eq 64
      end
    end
    context 'サムネイル高さのとき' do
      it 'デフォルトは50' do
        r = PettanImager.tmb_h
        r.should eq 50
      end
      it 'セットできる' do
        PettanImager.tmb_h = 64
        r = PettanImager.tmb_h
        r.should eq 64
      end
    end
  end
  
  describe 'バイナリデータの読み込みに於いて' do
    context 'つつがなく終わるとき' do
      it 'インスタンス化している' do
        r = PettanImager.load @d
        r.is_a?(PettanImager).should be_true
      end
      it 'バイナリデータを控えている' do
        r = PettanImager.load @d
        r.binary.should eq @d
      end
      it 'ライブラリを準備している' do
        r = PettanImager.load @d
        r.lib.should_not be_nil 
      end
    end
    context 'ロードに失敗したとき' do
      it 'falseを返す' do
        r = PettanImager.load 'error data'
        r.should be_false
      end
    end
  end
  describe 'バイナリデータの出力に於いて' do
    before do
      @i = PettanImager.load @d
    end
    context 'つつがなく終わるとき' do
      it '控えバイナリデータを返す' do
        r = @i.binary
        r.should eq @d
      end
    end
  end
  describe '幅の算出に於いて' do
    before do
      @i = PettanImager.load @d
      Magick::Image.any_instance.stub(:width).with(any_args).and_return(100)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに幅の取得を依頼している' do
        Magick::Image.any_instance.should_receive(:columns).with(any_args).exactly(1)
        r = @i.width
      end
      it 'データを返す' do
        r = @i.width
        r.should eq 100
      end
    end
  end
  describe '高さの算出に於いて' do
    before do
      @i = PettanImager.load @d
      Magick::Image.any_instance.stub(:height).with(any_args).and_return(200)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに高さの取得を依頼している' do
        Magick::Image.any_instance.should_receive(:rows).with(any_args).exactly(1)
        r = @i.height
      end
      it 'データを返す' do
        r = @i.height
        r.should eq 50
      end
    end
  end
  describe 'ファイルサイズの算出に於いて' do
    before do
      @i = PettanImager.load @d
      Magick::Image.any_instance.stub(:filesize).with(any_args).and_return(1500)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリにファイルサイズの取得を依頼している' do
        Magick::Image.any_instance.should_receive(:filesize).with(any_args).exactly(1)
        r = @i.filesize
      end
      it 'データを返す' do
        r = @i.filesize
        r.should eq 1500
      end
    end
  end
  describe '画像フォーマットの算出に於いて' do
    before do
      @i = PettanImager.load @d
      Magick::Image.any_instance.stub(:format).with(any_args).and_return('PNG')
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに画像フォーマットの取得を依頼している' do
        Magick::Image.any_instance.should_receive(:format).with(any_args).exactly(1)
        r = @i.ext
      end
      it 'データを小文字で返す' do
        r = @i.ext
        r.should eq 'png'
      end
    end
  end
  describe 'MD5の算出に於いて' do
    before do
      @i = PettanImager.load @d
    end
    context 'つつがなく終わるとき' do
      it 'バイナリデータをDigest::MD5に変換依頼している' do
        Digest::MD5.stub(:hexdigest).with(any_args).and_return('a'*32)
        Digest::MD5.should_receive(:hexdigest).with(any_args).exactly(1)
        r = @i.md5
      end
      it 'MD5を返す' do
        r = @i.md5
        r.size.should eq 32
      end
    end
  end
  describe '左右反転に於いて' do
    before do
      @i = PettanImager.load @d
      @mflip = Magick::Image.from_blob(@d).shift
      Magick::Image.any_instance.stub(:flip).with(any_args).and_return(@mflip)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに左右反転を依頼している' do
        Magick::Image.any_instance.should_receive(:flip).with(any_args).exactly(1)
        r = @i.v
      end
      it 'ライブラリに反転したバイナリデータの取得を依頼している' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@d)
        Magick::Image.any_instance.should_receive(:to_blob).with(any_args).exactly(1)
        r = @i.v
      end
      it 'バイナリデータを返す' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@mflip.to_blob)
        r = @i.v
        r.should eq @mflip.to_blob
      end
    end
  end
  describe '上下反転に於いて' do
    before do
      @i = PettanImager.load @d
      @mflop = Magick::Image.from_blob(@d).shift
      Magick::Image.any_instance.stub(:flop).with(any_args).and_return(@mflop)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに上下反転を依頼している' do
        Magick::Image.any_instance.should_receive(:flop).with(any_args).exactly(1)
        r = @i.h
      end
      it 'ライブラリに反転したバイナリデータの取得を依頼している' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@d)
        Magick::Image.any_instance.should_receive(:to_blob).with(any_args).exactly(1)
        r = @i.h
      end
      it 'バイナリデータを返す' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@mflop.to_blob)
        r = @i.h
        r.should eq @mflop.to_blob
      end
    end
  end
  describe '上下左右反転に於いて' do
    before do
      @i = PettanImager.load @d
      @mflip = Magick::Image.from_blob(@d).shift
      @mflop = Magick::Image.from_blob(@d).shift
      Magick::Image.any_instance.stub(:flip).with(any_args).and_return(@mflip)
      Magick::Image.any_instance.stub(:flop).with(any_args).and_return(@mflop)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリに上下左右反転を依頼している' do
        Magick::Image.any_instance.should_receive(:flip).with(any_args).exactly(1)
        Magick::Image.any_instance.should_receive(:flop).with(any_args).exactly(1)
        r = @i.vh
      end
      it 'ライブラリに反転したバイナリデータの取得を依頼している' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@d)
        Magick::Image.any_instance.should_receive(:to_blob).with(any_args).exactly(1)
        r = @i.vh
      end
      it 'バイナリデータを返す' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@mflop.to_blob)
        r = @i.vh
        r.should eq @mflop.to_blob
      end
    end
  end
  describe 'Gif変換に於いて' do
    before do
      @i = PettanImager.load @d
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリにGif変換を依頼している' do
        Magick::Image.any_instance.should_receive(:format=).with(any_args).exactly(1)
        r = @i.to_gif
      end
      it 'ライブラリにGif変換したバイナリデータの取得を依頼している' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@d)
        Magick::Image.any_instance.should_receive(:to_blob).with(any_args).exactly(1)
        r = @i.to_gif
      end
      it 'バイナリデータから画像処理オブジェクトを興している' do
        PettanImager.stub(:load).with(any_args).and_return(PettanImager.load(@d))
        PettanImager.should_receive(:load).with(any_args).exactly(1)
        r = @i.to_gif
      end
      it '画像処理オブジェクトを返す' do
        r = @i.to_gif
        r.is_a?(PettanImager).should be_true
      end
    end
  end
  describe 'サムネイル化に於いて' do
    before do
      @i = PettanImager.load @d
      @rs = Magick::Image.from_blob(@d).shift
      Magick::Image.any_instance.stub(:resize).with(any_args).and_return(@rs)
    end
    context 'つつがなく終わるとき' do
      it 'ライブラリにサムネイル化を依頼している' do
        Magick::Image.any_instance.should_receive(:resize).with(any_args).exactly(1)
        r = @i.to_thumbnail
      end
      it 'ライブラリにサムネイル化したバイナリデータの取得を依頼している' do
        Magick::Image.any_instance.stub(:to_blob).with(any_args).and_return(@d)
        Magick::Image.any_instance.should_receive(:to_blob).with(any_args).exactly(1)
        r = @i.to_thumbnail
      end
      it 'バイナリデータから画像処理オブジェクトを興す' do
        PettanImager.stub(:load).with(any_args).and_return(PettanImager.load(@d))
        PettanImager.should_receive(:load).with(any_args).exactly(1)
        r = @i.to_thumbnail
      end
      it '画像処理オブジェクトを返す' do
        r = @i.to_thumbnail
        r.is_a?(PettanImager).should be_true
      end
    end
  end
  describe 'サムネイルサイズの算出に於いて' do
    before do
      @i = PettanImager.load @d
    end
    context 'つつがなく終わるとき' do
      it '自身に幅の取得を依頼している' do
        PettanImager.any_instance.stub(:width).with(any_args).and_return(64)
        PettanImager.any_instance.should_receive(:width).with(any_args).exactly(1)
        r = @i.to_thumbnail
      end
      it '自身に高さの取得を依頼している' do
        PettanImager.any_instance.stub(:height).with(any_args).and_return(64)
        PettanImager.any_instance.should_receive(:height).with(any_args).exactly(1)
        r = @i.to_thumbnail
      end
      it '幅と高さの配列で返す' do
        r = @i.thumbnail_size
        r.is_a?(Array).should be_true
        r.size.should eq 2
      end
    end
  end
  
  describe 'サムネイルサイズの調整に於いて' do
    before do
      PettanImager.tmb_w=100
      PettanImager.tmb_h=200
    end
    context 'サムネイルサイズより小さいとき' do
      it 'そのまま返す' do
        r = PettanImager.thumbnail_size 100, 200
        r.should eq [100, 200]
      end
      it 'そのまま返す' do
        r = PettanImager.thumbnail_size 99, 199
        r.should eq [99, 199]
      end
    end
    context '幅が200%のとき' do
      it '幅がサイズに収まるように50%にして返す' do
        r = PettanImager.thumbnail_size 200, 200
        r.should eq [100, 100]
      end
    end
    context '高さが200%のとき' do
      it '高さがサイズに収まるように50%にして返す' do
        r = PettanImager.thumbnail_size 100, 400
        r.should eq [50, 200]
      end
    end
    context '幅が400%と高さが200%のとき' do
      it '幅がサイズに収まるように25%にして返す' do
        r = PettanImager.thumbnail_size 400, 400
        r.should eq [100, 100]
      end
    end
    context '幅が200%と高さが400%のとき' do
      it '高さがサイズに収まるように25%にして返す' do
        r = PettanImager.thumbnail_size 200, 800
        r.should eq [50, 200]
      end
    end
  end
  
end

