#サポートツール
require 'rubygems'
require 'active_support/all'
require 'rest_client'
require 'yaml'
require 'json'
require 'base64'

if ARGV.size < 1
  puts "uploader.rb host dirname auth"
  puts "upload on child dirs"
  exit
end

host = ARGV[0]
dirname = ARGV[1]
auth = ARGV[2]

class Uploader
  def initialize host, dirname, auth
    @host = host
    @dirname = dirname
    @auth = auth
    @licenses = self.licenses
    @license_groups = self.license_groups
  end
  
  def licenses
    res = {}
    u = @host + 'licenses.json'
    r = RestClient.get(u)
    ary = JSON.parse r
    ary.each do |l|
      res[l['name']] = l
    end
    res
  end

  def license_groups
    res = {}
    u = @host + 'license_groups.json'
    r = RestClient.get(u)
    ary = JSON.parse r
    ary.each do |l|
      res[l['id'].to_i] = l
    end
    res
  end
  
  def each_subdir
    Dir.glob File.expand_path(@dirname) + '/*' do |filename|
      if File.directory?(filename)
        yield filename
      end
    end
  end
  
  def each_opset 
    self.each_subdir do |filename|
      imgfile = nil
      lsname = nil
      formname = nil
      attr  = nil
      Dir.glob(filename + '/*') do |fn|
        ext = File.extname(fn).downcase
        case ext
        when '.json'
          json = JSON.parse(File.open(fn).read)
          lsname = json["license_name"]
          formname = json["formname"]
          attr = json["attributes"]
        when '.png', '.gif', '.jpeg'
          imgfile = fn
        end
      end
      if imgfile and lsname and formname and attr
        yield imgfile, lsname, formname, attr
      end
    end
  end
  
  def upload fn
    oid = nil
    b = Base64.encode64(File.open(fn, 'rb').read)
    u = @host + 'original_pictures'
    begin
      r = RestClient.post(u, 
        {:original_picture => {:file  => b}, :auth_token => @auth}.to_json, 
        :content_type => :json, :accept => :json
      )
      o = JSON.parse r
      oid = o['id']
    rescue
      puts ' failed: ' + r.to_s
    end
    oid
  end
  
  def resource_picture oid, lgid, lsid, en, formname, attr
    rp = nil
    u = @host + en.tableize + '/attributes'
    attr['license_id'] = lsid
    param = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :auth_token => @auth 
    }
    param[formname] = attr
    begin
      r = RestClient.post(u, 
        param.to_json,
        :content_type => :json, :accept => :json
      )
      rp = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
    rp
  end
  
  def publish oid, lgid, rp
    res = nil
    u = @host + 'resource_pictures'
    attr = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :resource_picture => {
        :original_picture_id => oid, 
        :license_id => rp['license_id'], 
        :artist_name => rp['artist_name'], 
        :classname => rp['classname'], 
        :credit => rp['credit'], 
        :settings => rp['settings']
      }, 
      :auth_token => @auth 
    }
    begin
      r = RestClient.post(u, 
        attr.to_json,
        :content_type => :json, :accept => :json
      )
      res = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
    res
  end
  
  def publish_all
    each_opset do |imgfile, lsname, formname, attr|
      puts 'uploading: ' + imgfile
      oid = self.upload imgfile
      next unless oid
      puts 'success: ' + oid.to_s
      ls = @licenses[lsname]
      lsid = ls['id']
      lgid = ls['license_group_id'].to_i
      lg = @license_groups[lgid]
      en = lg['classname']
      puts 'building: ' + formname
      rp = self.resource_picture oid, lgid, lsid, en, formname, attr
      next unless rp
      puts 'publishing: ' + lsname
      r = self.publish oid, lgid, rp
      puts 'success: ' + r['id'].to_s
    end
  end
  
end

u = Uploader.new host, dirname, auth

u.publish_all

